Attribute VB_Name = "SubMain"
Option Explicit

' Copyright 2000-2003, Rocky Mountain Computer Consulting, Inc.
'
' Feel free to use this program youself or distribute it
' in its unchanged form. Better than distributing it,
' tell people to get it from:
'
'       http://www.vb-helper.com/howto_picture_saver.html
'
' This page contains the most recent version of the program.
'
' See the file Readme.txt for installation instructions and
' other information.

Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Public Const SWP_NOACTIVATE = &H10
Public Const SWP_NOZORDER = &H4
Public Const SWP_SHOWWINDOW = &H40

Public Const HWND_TOP = 0

Public Const WS_CHILD = &H40000000
Public Const GWL_HWNDPARENT = (-8)
Public Const GWL_STYLE = (-16)

Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long) As Long
Declare Function GetClientRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Declare Function SetParent Lib "user32" (ByVal hWndChild As Long, ByVal hWndNewParent As Long) As Long
Declare Function SetWindowPos Lib "user32" (ByVal hwnd As Long, ByVal hWndInsertAfter As Long, ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long
Declare Function ShowCursor Lib "user32" (ByVal bShow As Long) As Long
Declare Function FindWindow Lib "user32" Alias "FindWindowA" (ByVal lpClassName As String, ByVal lpWindowName As String) As Long

' Global variables.
Public Enum RunModes
    rmConfigure
    rmScreenSaver
    rmPreview
End Enum
Public RunMode As RunModes

' The directories we will use.
Public g_Directories As Collection

' The files we will use.
Public g_Files As Collection

' The application name for the registry.
Public Const APP_NAME = "PictureSaver"

' Should we display file names?
Public g_ShowFileNames As Boolean

' Seconds between pictures.
Public g_Interval As Long

' The next time we should display an image.
Public g_NextTime As Date
' See if another instance of the program is
' running in screen saver mode.
Private Sub CheckShouldRun()
Const CAP = APP_NAME & "/S"

    ' If no instance is running, we're safe.
    If Not App.PrevInstance Then Exit Sub

    ' See if there is a screen saver mode instance.
    If FindWindow(vbNullString, CAP) Then End

    ' Set our caption so other instances can find
    ' us in the previous line.
    frmCover.Caption = CAP
End Sub
' Load configuration information from the registry.
Public Sub LoadConfig()
Dim num_dirs As Integer
Dim dir_name As String
Dim i As Integer

    ' See how many directories there are.
    num_dirs = CInt(GetSetting(APP_NAME, "Settings", "NumDirs", "0"))

    ' Get the directory names and the files they contain.
    Set g_Directories = New Collection
    Set g_Files = New Collection
    For i = 1 To num_dirs
        dir_name = GetSetting(APP_NAME, "Settings", "Dir" & Format$(i), "")
        If Len(dir_name) > 0 Then
            g_Directories.Add dir_name
            LoadDirectoryFiles dir_name
        End If
    Next i

    ' Get the interval.
    g_Interval = CLng(GetSetting(APP_NAME, "Settings", "Interval", "60"))

    ' Should we show file names?
    g_ShowFileNames = CBool(GetSetting(APP_NAME, "Settings", "ShowFileNames", "False"))
End Sub
' Load the files in this directory into g_Files.
Public Sub LoadDirectoryFiles(ByVal dir_name As String)
Dim patterns As Variant
Dim file_name As String
Dim i As Integer

    ' Make sure the directory name ends with "\".
    If Right$(dir_name, 1) <> "\" Then dir_name = dir_name & "\"

    ' Get the files in the directory.
    patterns = Array("*.bmp", "*.jpg", "*.jpeg", "*.gif")
    For i = LBound(patterns) To UBound(patterns)
        ' Get files that match the pattern.
        file_name = Dir$(dir_name & patterns(i))
        Do While Len(file_name) > 0
            g_Files.Add dir_name & file_name
            file_name = Dir$()
        Loop
    Next i
End Sub
' Save configuration information in the registry.
Public Sub SaveConfig()
Dim old_num_dirs As Integer
Dim i As Integer

    ' Get the old number of directories saved.
    old_num_dirs = CInt(GetSetting(APP_NAME, "Settings", "NumDirs", "0"))

    ' Save the new number of directories.
    SaveSetting APP_NAME, "Settings", "NumDirs", Format$(g_Directories.Count)

    ' Save the directory names.
    For i = 1 To g_Directories.Count
        SaveSetting APP_NAME, "Settings", "Dir" & Format$(i), g_Directories(i)
    Next i

    ' Remove any extra old entries.
    For i = g_Directories.Count + 1 To old_num_dirs
        DeleteSetting APP_NAME, "Settings", "Dir" & Format$(i)
    Next i

    ' Save the interval.
    SaveSetting APP_NAME, "Settings", "Interval", Format$(g_Interval)

    ' Should we show file names?
    SaveSetting APP_NAME, "Settings", "ShowFileNames", Format$(g_ShowFileNames)
End Sub
' Start the program.
Public Sub Main()
Dim args As String
Dim preview_hwnd As Long
Dim preview_rect As RECT
Dim window_style As Long

    Randomize

    ' Get the configuration parameters.
    LoadConfig

    ' Get the command line arguments.
    args = UCase$(Trim$(Command$))

    ' Examine the first 2 characters.
    Select Case Mid$(args, 1, 2)
        Case "/C"   ' Display configuration dialog.
            RunMode = rmConfigure
        Case "", "/S"   ' Run as a screen saver.
            RunMode = rmScreenSaver
        Case "/P"       ' Run in preview mode.
            RunMode = rmPreview
        Case Else       ' This shouldn't happen.
            RunMode = rmScreenSaver
    End Select

    ' Run in the appropriate mode.
    Select Case RunMode
        Case rmConfigure    ' Display configuration dialog.
            frmConfig.Show

        Case rmScreenSaver  ' Run as a screen saver.
            ' Make sure there isn't another one running.
            CheckShouldRun

            ' Display the cover form.
            frmCover.Show

            ' Force maximized (for Windows 2000).
            frmCover.WindowState = vbMaximized
            ShowCursor False

        Case rmPreview      ' Run in preview mode.
            ' Get the preview area hWnd.
            preview_hwnd = GetHwndFromCommand(args)

            ' Get the dimensions of the preview area.
            GetClientRect preview_hwnd, preview_rect

            Load frmCover

            ' Set the caption for Windows 95.
            frmCover.Caption = "Preview"

            ' Add WS_CHILD to make this a child window.
            window_style = GetWindowLong(frmCover.hwnd, GWL_STYLE) Or WS_CHILD
            SetWindowLong frmCover.hwnd, _
                GWL_STYLE, window_style

            ' Set the window's parent so it appears
            ' inside the preview area.
            SetParent frmCover.hwnd, preview_hwnd

            ' Save the preview area's hWnd in
            ' the form's window structure.
            SetWindowLong frmCover.hwnd, _
                GWL_HWNDPARENT, preview_hwnd

            ' Show the preview.
            SetWindowPos frmCover.hwnd, _
                HWND_TOP, 0&, 0&, _
                preview_rect.Right, _
                preview_rect.Bottom, _
                SWP_NOZORDER Or SWP_NOACTIVATE Or SWP_SHOWWINDOW
    End Select
End Sub
' Get the hWnd for the preview window from the
' command line arguments.
Private Function GetHwndFromCommand(ByVal args As String) As Long
Dim argslen As Integer
Dim i As Integer
Dim ch As String

    ' Take the rightmost numeric characters.
    args = Trim$(args)
    argslen = Len(args)
    For i = argslen To 1 Step -1
        ch = Mid$(args, i, 1)
        If ch < "0" Or ch > "9" Then Exit For
    Next i

    GetHwndFromCommand = CLng(Mid$(args, i + 1))
End Function

